/*****************************************************************


			F O R W A R D   P U B L I C S


*****************************************************************/
OnPluginStart_ModelList()
{
	SetupDefaultModelList();
	
	RegAdminCmd("sm_modellist", Command_ModelList, ADMFLAG_ROOT, "sm_modellist - Print loaded modelfun model list to console");
	RegAdminCmd("sm_refreshmodels", Command_RefreshModels, ADMFLAG_ROOT, "sm_refreshmodels - Reloads modelfun model list from 'addons/sourcemod/data/modelfun.txt'");
	RegAdminCmd("sm_addmodel", Command_AddModel, ADMFLAG_ROOT, "sm_addmodel <model name> <model file path> - Add a model to the modelfun model list");
	RegAdminCmd("sm_deletemodel", Command_DeleteModel, ADMFLAG_ROOT, "sm_deletemodel <model name> -  Remove a model from the modelfun model list");
}




/****************************************************************


			C A L L B A C K   F U N C T I O N S


****************************************************************/
public Action:Command_ModelList(client, args)
{
	if( !PrintList( client, "modellist" ) )
	{
		PrintToConsole( client, "No Models loaded!" );
	}
	
	return Plugin_Handled;		
}


public Action:Command_RefreshModels(client, args)
{
	if(KvJumpToKey(hKVModelFun, "modellist"))
	{
		KvDeleteThis( hKVModelFun );
		KvRewind(hKVModelFun);
		
		if(FileToKeyValues(hKVModelFun, g_modelfunKV))
		{
			ReplyToCommand(client, "[SM] Modelfun model list reloaded!");

			KvRewind(hKVModelFun);
	
			return Plugin_Handled;
		}
	}
	
	ReplyToCommand(client, "[SM] Error reloading modelfun model list");
	KvRewind(hKVModelFun);
	
	return Plugin_Handled;	
}


public Action:Command_AddModel(client, args)
{
	decl String:bufferKey[49];
	decl String:bufferValue[129];
	
	//not enough arguments, display usage
	if (args < 2)
	{
		ReplyToCommand(client, "[SM] Usage: sm_addmodel <model name> <model file path>");
		return Plugin_Handled;
	}	

	//get command arguments
	GetCmdArg(1, bufferKey, sizeof(bufferKey));
	GetCmdArg(2, bufferValue, sizeof(bufferValue));

	//set model join in kv file	
	if(KvJumpToKey(hKVModelFun, "modellist"))
	{
		//make sure key does not already exist
		if( KvJumpToKey(hKVModelFun, bufferKey, false) )
		{
			KvRewind(hKVModelFun);
			ReplyToCommand(client, "[SM] Model '%s' already exists!", bufferKey);
			
			return Plugin_Handled;	
		}
		
		//set, rewind and save
		KvSetString(hKVModelFun, bufferKey, bufferValue );

		KvRewind(hKVModelFun);
		KeyValuesToFile(hKVModelFun, g_modelfunKV);
		
		ReplyToCommand(client, "[SM] Added model '%s' (%s) to model list", bufferKey, bufferValue);
		
		return Plugin_Handled;		
	}

	//couldnt get modellist root key
	KvRewind(hKVModelFun);
	
	ReplyToCommand(client, "[SM] Error adding model");
	
	return Plugin_Handled;
}


public Action:Command_DeleteModel(client, args)
{
	decl String:bufferKey[49];
	
	//not enough arguments, display usage
	if (args < 1)
	{
		ReplyToCommand(client, "[SM] Usage: sm_deletemodel <model name> ");
		return Plugin_Handled;
	}	

	//get command arguments
	GetCmdArg(1, bufferKey, sizeof(bufferKey));

	//set model join in kv file	
	if(KvJumpToKey(hKVModelFun, "modellist"))
	{
		//make sure key exists
		if( !KvJumpToKey(hKVModelFun, bufferKey, false) )
		{
			KvRewind(hKVModelFun);
			ReplyToCommand(client, "[SM] Model '%s' does not exist!", bufferKey);
			
			return Plugin_Handled;	
		}
		
		//delete, rewind and save
		KvDeleteThis(hKVModelFun);
		KvRewind(hKVModelFun);
		KeyValuesToFile(hKVModelFun, g_modelfunKV);
		
		ReplyToCommand(client, "[SM] Removed model '%s' from the model list", bufferKey);
		
		return Plugin_Handled;		
	}

	//couldnt get modellist root key
	KvRewind(hKVModelFun);
	
	ReplyToCommand(client, "[SM] Error deleting model");
	
	return Plugin_Handled;
}


/*****************************************************************


			P L U G I N   F U N C T I O N S


*****************************************************************/
SetupDefaultModelList()
{
	if(!KvJumpToKey(hKVModelFun, "modellist"))
	{				
		KvJumpToKey(hKVModelFun, "modellist", true);
		KvSetString(hKVModelFun, "HL2 Horse", 	"models/props_c17/statue_horse.mdl");
		KvSetString(hKVModelFun, "HL2 Gman", 	"models/gman.mdl");
		KvSetString(hKVModelFun, "HL2 Dog", 	"models/dog.mdl");
		KvSetString(hKVModelFun, "HL2 Stalker", "models/stalker.mdl");
		KvSetString(hKVModelFun, "HL2 Gunship", "models/gunship.mdl");
		KvSetString(hKVModelFun, "HL2 Pigeon", 	"models/pigeon.mdl");
		KvSetString(hKVModelFun, "HL2 Citadel", "models/props_combine/combine_citadel_animated.mdl");
		

		KvRewind(hKVModelFun);			
		KeyValuesToFile(hKVModelFun, g_modelfunKV);
		
		return true;		
	}
	else
	{
		KvRewind(hKVModelFun);		
		return false;
	}	
}