//admin menu type constants
#define MODELMENU_PHYSICS   		"0"
#define MODELMENU_MODEL				"1"

#define MODELMENU_PHYSICS_TITLE   		"Spawn Physics Model"
#define MODELMENU_MODEL_TITLE			"Spawn Static Model"

/*****************************************************************


			G L O B A L   V A R S


*****************************************************************/
new Handle:h_arraySpawnedItems = INVALID_HANDLE;


/*****************************************************************


			F O R W A R D   P U B L I C S


*****************************************************************/
OnPluginStart_Model()
{
	h_arraySpawnedItems = CreateArray();
	
	RegAdminCmd("sm_removemodel", Command_RemoveModel, ADMFLAG_ROOT, "sm_removemodel - Removes last spawned entity");
	RegAdminCmd("sm_removemodels", Command_RemoveModels, ADMFLAG_ROOT, "sm_removemodels - Removes ALL spawned entities");
	RegAdminCmd("sm_spawnmodel", Command_SpawnModel, ADMFLAG_ROOT, "sm_spawnmodel \"model name\" - Spawns a prop_dynamic_override entity with the supplied model");
	RegAdminCmd("sm_spawnphysics", Command_SpawnPhysics, ADMFLAG_ROOT, "sm_spawnmodel \"model name\" - Spawns a prop_physics_multiplayer entity with the supplied model");
}

OnMapStart_Model()
{	
	ClearArray( h_arraySpawnedItems );
}

OnPluginEnd_Model()
{
	RemoveAllEntities();	
}

/****************************************************************


			C A L L B A C K   F U N C T I O N S


****************************************************************/
public Action:Command_SpawnModel(client, args)
{
	decl String:modelName[129];
	
	//validate args
	if (args < 1)
	{
		ReplyToCommand(client, "[SM] Usage: sm_spawnmodel \"model name\"");
		return Plugin_Handled;
	}
	
	if( !client )
	{
		ReplyToCommand(client, "[SM] Cannot create entity from rcon");
		return Plugin_Handled;	
	}
	
	//get argument
	GetCmdArg(1, modelName, sizeof(modelName));

	if(!CreateEntity(client,"prop_dynamic_override", modelName))
	{
		ReplyToCommand(client, "[SM] Cannot create model \"%s\"", modelName);
	}
	
	ShowActivity2(client, "[SM] ", "\"%L\" spawned static model \"%s\"", client, modelName);

	return Plugin_Handled;	
}

public Action:Command_SpawnPhysics(client, args)
{
	decl String:modelName[129];
	
	//validate args
	if (args < 1)
	{
		ReplyToCommand(client, "[SM] Usage: sm_spawnphysics \"model name\"");
		return Plugin_Handled;
	}
	
	if( !client )
	{
		ReplyToCommand(client, "[SM] Cannot create entity from rcon");
		return Plugin_Handled;	
	}
	
	//get argument
	GetCmdArg(1, modelName, sizeof(modelName));

	if(!CreateEntity(client,"prop_physics_override", modelName))
	{
		ReplyToCommand(client, "[SM] Cannot create model \"%s\"", modelName);
	}
	
	ShowActivity2(client, "[SM] ", "\"%L\" spawned physics model \"%s\"", client, modelName);

	return Plugin_Handled;	
}


public Action:Command_RemoveModel(client, args)
{
	if( RemoveEntityLast() )
	{
		ReplyToCommand(client, "[SM] Entity removed");
	}
	else
	{
		ReplyToCommand(client, "[SM] No entity to remove!");
	}
	
	return Plugin_Handled;	
}

public Action:Command_RemoveModels(client, args)
{
	RemoveAllEntities();
	ReplyToCommand(client, "[SM] All entities removed");

	return Plugin_Handled;	
}






/*****************************************************************


			P L U G I N   F U N C T I O N S


*****************************************************************/
bool:CreateEntity(client, const String:entType[],String:modelName[129])
{
	new prop = -1;
	
	//check max spawnables and remove if necesary
	if( GetArraySize(h_arraySpawnedItems) >= g_maxSpawns )
	{
		RemoveEntityFirst();
	}
	
	decl Float:Endpoint[3], Float:Angles[3], Float:Startpoint[3], Float:AngleVec[3];
	new Float:Distance = 270.0;
	
	//cache model if necesary
	if (!IsModelPrecached(modelName))
	{
    	if(!PrecacheModel(modelName))
    	{
    		return false;
    	}
    }

	//get spawn pos	
	GetClientEyeAngles(client, Angles);
	GetClientAbsOrigin(client, Startpoint);
	GetAngleVectors(Angles, AngleVec, NULL_VECTOR, NULL_VECTOR);
	
	//calc endpoint
	Endpoint[0] = Startpoint[0] + (AngleVec[0]*Distance);
	Endpoint[1] = Startpoint[1] + (AngleVec[1]*Distance);
	Endpoint[2] = Startpoint[2] + (AngleVec[2]*Distance);
	
	//create new entity
	prop = CreateEntityByName(entType);
    
	if (IsValidEntity(prop)) 
	{
		DispatchKeyValue(prop, "model", modelName);
		DispatchKeyValue(prop, "solid", "6");
		DispatchKeyValueVector(prop, "angles", Angles);
		
		DispatchSpawn(prop);
		AcceptEntityInput(prop, "TurnOn", prop, prop, 0);
		
		TeleportEntity(prop, Endpoint, NULL_VECTOR, NULL_VECTOR);
		
		PushArrayCell( h_arraySpawnedItems, prop );
	}
	else
	{
		return false;
	}
	
	LogAction(client, -1, "\"%L\" spawned entity type \"%s\" with model \"%s\"", client, entType, modelName);
	
	return true;
}

bool:RemoveEntityFirst()
{
	if( GetArraySize(h_arraySpawnedItems) > 0)
	{
		if( IsValidEntity( GetArrayCell( h_arraySpawnedItems, 0 ) ) )
		{
			RemoveEdict(GetArrayCell( h_arraySpawnedItems, 0 ) );
		}
		
		RemoveFromArray( h_arraySpawnedItems, 0 );
		
		return true;
	}
	
	return false;
}

bool:RemoveEntityLast()
{
	if( GetArraySize(h_arraySpawnedItems) > 0)
	{
		if( IsValidEntity( GetArrayCell( h_arraySpawnedItems, GetArraySize(h_arraySpawnedItems) - 1 ) ) )
		{
			RemoveEdict(GetArrayCell( h_arraySpawnedItems, GetArraySize(h_arraySpawnedItems) - 1 ) );
		}

		RemoveFromArray( h_arraySpawnedItems, GetArraySize(h_arraySpawnedItems) - 1 );

		return true;
	}
	
	return false;
}

RemoveAllEntities()
{
	for( new i = 0; i < GetArraySize(h_arraySpawnedItems); i++ )
	{		
		CreateTimer(0.1, Timer_Event_RemoveEntity);
	}
}

public Action:Timer_Event_RemoveEntity(Handle:timer) 
{
	if( IsValidEntity( GetArrayCell( h_arraySpawnedItems, 0 ) ) )
	{
		RemoveEdict(GetArrayCell( h_arraySpawnedItems, 0 ) );
	}
	
	RemoveFromArray( h_arraySpawnedItems, 0 );
	
	return Plugin_Handled;
}



/*****************************************************************


			A D M I N   M E N U   F U N C T I O N S


*****************************************************************/

SetupAdminMenu_Model_Physics(TopMenuObject:parentmenu)
{
	AddToTopMenu(hTopMenu, 
		"sm_spawnphysics",
		TopMenuObject_Item,
		AdminMenu_Model,
		parentmenu,
		"sm_spawnphysics",
		ADMFLAG_ROOT,
		MODELMENU_PHYSICS);
}

SetupAdminMenu_Model_Static(TopMenuObject:parentmenu)
{
	AddToTopMenu(hTopMenu, 
		"sm_spawnmodel",
		TopMenuObject_Item,
		AdminMenu_Model,
		parentmenu,
		"sm_spawnmodel",
		ADMFLAG_ROOT,
		MODELMENU_MODEL);
}

public AdminMenu_Model(Handle:topmenu, 
					  TopMenuAction:action,
					  TopMenuObject:object_id,
					  param,
					  String:buffer[],
					  maxlength)
{	
	decl String:modelMenuType[33];
	decl String:title[100];
	
	//detect model menu type
	GetTopMenuInfoString( topmenu, object_id, modelMenuType, sizeof(modelMenuType) );
	if( strcmp( modelMenuType, MODELMENU_PHYSICS ) == 0 )
	{
		Format(title, sizeof(title), "%s", MODELMENU_PHYSICS_TITLE);
	}
	else if( strcmp( modelMenuType, MODELMENU_MODEL ) == 0 )
	{
		Format(title, sizeof(title), "%s", MODELMENU_MODEL_TITLE);
	}

	if (action == TopMenuAction_DisplayOption)
	{
		Format(buffer, maxlength, "%s", title);
	}
	else if (action == TopMenuAction_SelectOption)
	{
		DisplayMenu_Model(param, title);
	}
}

DisplayMenu_Model(client, const String:modelMenuTitle[])
{
	new Handle:menu = CreateMenu(MenuHandler_Model);
		
	SetMenuTitle(menu, modelMenuTitle);
	SetMenuExitBackButton(menu, true);
	
	AddModelsToMenu(menu, "modellist");
	
	DisplayMenu(menu, client, MENU_TIME_FOREVER);
}

public MenuHandler_Model(Handle:menu, MenuAction:action, param1, param2)
{
	decl String:title[100];
	
	GetMenuTitle(menu, title, sizeof(title));
	
	if (action == MenuAction_End)
	{
		CloseHandle(menu);
	}
	else if (action == MenuAction_Cancel)
	{
		if (param2 == MenuCancel_ExitBack && hTopMenu != INVALID_HANDLE)
		{
			DisplayTopMenu(hTopMenu, param1, TopMenuPosition_LastCategory);
		}
	}
	else if (action == MenuAction_Select)
	{
		decl String:modelName[129];
		GetMenuItem(menu, param2, modelName, sizeof(modelName));
				
		if( strcmp( title, MODELMENU_PHYSICS_TITLE ) == 0 )
		{
			if(!CreateEntity(param1,"prop_physics_override", modelName))
			{
				ReplyToCommand(param1, "[SM] Cannot create model \"%s\"", modelName);
			}
			
			ShowActivity2(param1, "[SM] ", "\"%L\" spawned physics model \"%s\"", param1, modelName);
		}
		else if( strcmp( title, MODELMENU_MODEL_TITLE ) == 0 )
		{
			if(!CreateEntity(param1,"prop_dynamic_override", modelName))
			{
				ReplyToCommand(param1, "[SM] Cannot create model \"%s\"", modelName);
			}
			
			ShowActivity2(param1, "[SM] ", "\"%L\" spawned static model \"%s\"", param1, modelName);
		}
		
		DisplayMenu_Model(param1, title);
	}
	
	return;
}



SetupAdminMenu_Model_RemoveAll(TopMenuObject:parentmenu)
{
	AddToTopMenu(hTopMenu,
		"sm_removemodels",
		TopMenuObject_Item,
		AdminMenu_Model_RemoveAll,
		parentmenu,
		"sm_removemodels",
		ADMFLAG_ROOT);
}


public AdminMenu_Model_RemoveAll(Handle:topmenu, 
					  TopMenuAction:action,
					  TopMenuObject:object_id,
					  param,
					  String:buffer[],
					  maxlength)
{
	if (action == TopMenuAction_DisplayOption)
	{
		Format(buffer, maxlength, "%s", "Remove Spawned Models");
	}
	else if (action == TopMenuAction_SelectOption)
	{
		
		RemoveAllEntities();
		ReplyToCommand(param, "[SM] All entities removed");
		
		RedisplayAdminMenu(topmenu, param);	
	}
}